@@ -28,6 +28,7 @@
 #include <signal.h>
 #include <gdk/gdkkeysyms.h>
 #include <math.h>
+
 #include "procman.h"
 #include "callbacks.h"
 #include "interface.h"
@@ -36,9 +37,10 @@
 #include "procactions.h"
 #include "load-graph.h"
 #include "cellrenderer.h"
+#include "util.h"
 
-void	cb_toggle_tree (GtkMenuItem *menuitem, gpointer data);
-void	cb_toggle_threads (GtkMenuItem *menuitem, gpointer data);
+static void	cb_toggle_tree (GtkMenuItem *menuitem, gpointer data);
+static void	cb_toggle_threads (GtkMenuItem *menuitem, gpointer data);
 
 
 static GnomeUIInfo file1_menu_uiinfo[] =
@@ -122,42 +124,40 @@ static GnomeUIInfo popup_menu_uiinfo[] =
 {
 	{
  	  GNOME_APP_UI_ITEM, N_("_Change Priority..."), N_("Change the importance (nice value) of a process"),
-	 popup_menu_renice, NULL, NULL, 0, NULL,
+	 cb_renice, NULL, NULL, 0, NULL,
 	 0, 0
 	},
 	{
  	  GNOME_APP_UI_ITEM, N_("_Memory Maps"), N_("View the memory maps associated with a process"),
-	 popup_menu_show_memory_maps, NULL, NULL, 0, NULL,
+	 cb_show_memory_maps, NULL, NULL, 0, NULL,
 	 0, 0
 	},
 	GNOMEUIINFO_SEPARATOR,
 	{
  	  GNOME_APP_UI_ITEM, N_("_Hide Process"), N_("Hide a process"),
-	 popup_menu_hide_process, NULL, NULL, 0, NULL,
+	 cb_hide_process, NULL, NULL, 0, NULL,
 	 0, 0
 	},
 	GNOMEUIINFO_SEPARATOR,
 	{
  	  GNOME_APP_UI_ITEM, N_("_End Process"), N_("Force a process to finish"),
-	 popup_menu_end_process, NULL, NULL, 0, NULL,
+	 cb_end_process, NULL, NULL, 0, NULL,
 	 0, 0
 	},
 	{
  	  GNOME_APP_UI_ITEM, N_("_Kill Process"), N_("Force a process to finish now"),
-	 popup_menu_kill_process, NULL, NULL, 0, NULL,
+	 cb_kill_process, NULL, NULL, 0, NULL,
 	 0, 0
 	},
 	GNOMEUIINFO_END
 };
 
-gchar *moreinfolabel = N_("More _Info >>");
-gchar *lessinfolabel = N_("<< Less _Info");
 
-GtkWidget *infobutton;
-GtkWidget *infolabel;
-GtkWidget *endprocessbutton;
-GtkWidget *popup_menu;
-GtkWidget *sys_pane;
+static GtkWidget *infolabel = NULL;
+static GtkWidget *endprocessbutton = NULL;
+static GtkWidget *popup_menu = NULL;
+static GtkWidget *sys_pane = NULL;
+
 
 gint
 get_sys_pane_pos (void)
@@ -177,6 +177,7 @@ create_proc_view (ProcData *procdata)
 	GtkWidget *infobox;
 	GtkWidget *label;
 	GtkWidget *hbox2;
+	GtkWidget *infobutton;
 
 	vbox1 = gtk_vbox_new (FALSE, 18);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 12);
@@ -255,36 +256,26 @@ sort_bytes (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointe
 {
 	int col = GPOINTER_TO_INT (data);
 	float btotal1, btotal2, bfree1, bfree2;
-	float a, b;
-	
-	btotal1 = btotal2 = bfree1 = bfree2 = 0.0;
-	gtk_tree_model_get (model, itera, 7, &btotal1, 8, &bfree1, -1);
-	gtk_tree_model_get (model, iterb, 7, &btotal2, 8, &bfree2, -1);
+
+	btotal1 = btotal2 = bfree1 = bfree2 = 0.0f;
+	gtk_tree_model_get (model, itera, 8, &btotal1, 9, &bfree1, -1);
+	gtk_tree_model_get (model, iterb, 8, &btotal2, 9, &bfree2, -1);
 
 	switch (col)
 	{
-		case 4: 
-			a = btotal1;
-			b = btotal2;
-			break;
-		case 5: 
-			a = btotal1 - bfree1;
-			b = btotal2 - bfree2;
-			break;
-		case 6: 
-			a = bfree1;
-			b = bfree2;
-			break;
-		default:
-			a = 0;
-			b = 0;
-	}
+		case 4:
+			return PROCMAN_CMP(btotal1, btotal2);
 
-	if (a > b)
-		return -1;
-	else if (a < b)
-		return 1;
-	return 0;	
+		case 5:
+			return PROCMAN_CMP(bfree1, bfree2);
+
+		case 6:
+			return PROCMAN_CMP(btotal1 - bfree1, btotal2 - bfree2);
+
+		default:
+			g_assert_not_reached();
+			return 0;
+	}
 }
 
 static GtkWidget *
@@ -307,23 +298,20 @@ make_title_label (const char *text)
 static void
 cpu_size_request (GtkWidget *box, GtkRequisition *req, ProcData *procdata)
 {
-	if (req->width > procdata->cpu_label_fixed_width)
-		procdata->cpu_label_fixed_width = req->width;
-		
-	req->width = procdata->cpu_label_fixed_width;
-	
+	req->width = procdata->cpu_label_fixed_width = \
+		MAX(req->width, procdata->cpu_label_fixed_width);
 }
 
 static GtkWidget *
 create_sys_view (ProcData *procdata)
 {
-	GtkWidget *vbox;
+	GtkWidget *vbox, *hbox;
 	GtkWidget *vpane;
 	GtkWidget *cpu_box, *mem_box, *disk_box;
 	GtkWidget *cpu_hbox, *mem_hbox, *disk_hbox;
 	GtkWidget *cpu_graph_box, *mem_graph_box;
 	GtkWidget *label,*cpu_label, *spacer;
-	GtkWidget *hbox, *table;
+	GtkWidget *table;
 	GtkWidget *color_picker;
 	GtkWidget *scrolled;
 	GtkWidget *disk_tree;
@@ -331,14 +319,20 @@ create_sys_view (ProcData *procdata)
 	GtkTreeViewColumn *col;
 	GtkCellRenderer *cell;
 	GtkSizeGroup *sizegroup;
-	gchar *titles[5] = {_("Name"),
-			    _("Directory"),
-				_("Type"),
-			    _("Total"),
-			    _("Used"),
-				};
+
+	static const gchar *titles[] = {
+	  N_("Device"),
+	  N_("Directory"),
+	  N_("Type"),
+	  N_("Total"),
+	  N_("Free"),
+	  N_("Used"),
+	};
+
 	LoadGraph *cpu_graph, *mem_graph;
 	gint i;
+
+	PROCMAN_GETTEXT_ARRAY_INIT(titles);
 	
 	vpane = gtk_vpaned_new ();
 	sys_pane = vpane;
@@ -369,18 +363,17 @@ create_sys_view (ProcData *procdata)
 	cpu_graph_box = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (cpu_hbox), cpu_graph_box, TRUE, TRUE, 0);
 
-	cpu_graph = load_graph_new (CPU_GRAPH, procdata);
+	cpu_graph = load_graph_new (LOAD_GRAPH_CPU, procdata);
 	gtk_box_pack_start (GTK_BOX (cpu_graph_box), cpu_graph->main_widget, TRUE, TRUE, 0);
 
 	sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (cpu_graph_box), hbox, FALSE, FALSE, 0);
+
 	for (i=0;i<procdata->config.num_cpus; i++) {
 		GtkWidget *temp_hbox;
 		gchar *text;
-		/* Two per row */
-		if (fabs(fmod(i,2) - 0) < .01) {
-			hbox = gtk_hbox_new (FALSE, 12);
-			gtk_box_pack_start (GTK_BOX (cpu_graph_box), hbox, FALSE, FALSE, 0);
-		}
 		
 		temp_hbox = gtk_hbox_new (FALSE, 6);
 		gtk_box_pack_start (GTK_BOX (hbox), temp_hbox, FALSE, FALSE, 0);
@@ -430,11 +423,11 @@ create_sys_view (ProcData *procdata)
 	gtk_box_pack_start (GTK_BOX (mem_hbox), mem_graph_box, TRUE, TRUE, 0);
 
 
-	mem_graph = load_graph_new (MEM_GRAPH, procdata);
+	mem_graph = load_graph_new (LOAD_GRAPH_MEM, procdata);
 	gtk_box_pack_start (GTK_BOX (mem_graph_box), mem_graph->main_widget, 
 			    TRUE, TRUE, 0);
 	
-	table = gtk_table_new (2, 6, FALSE);
+	table = gtk_table_new (2, 7, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 	gtk_box_pack_start (GTK_BOX (mem_graph_box), table, 
@@ -467,6 +460,11 @@ create_sys_view (ProcData *procdata)
 	gtk_table_attach (GTK_TABLE (table), mem_graph->memtotal_label, 4, 5, 0, 1, 
 			  GTK_FILL, 0, 0, 0);
 	
+	mem_graph->mempercent_label = gtk_label_new ("");
+	gtk_misc_set_alignment (GTK_MISC (mem_graph->mempercent_label), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), mem_graph->mempercent_label, 5, 6, 0, 1, 
+			  GTK_FILL, 0, 0, 0);
+			  
 	color_picker = gnome_color_picker_new ();
 	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (color_picker), 
 				    mem_graph->colors[3].red,
@@ -494,6 +492,11 @@ create_sys_view (ProcData *procdata)
 	gtk_table_attach (GTK_TABLE (table), mem_graph->swaptotal_label, 4, 5, 1, 2, 
 			  GTK_FILL, 0, 0, 0);	
 	
+	mem_graph->swappercent_label = gtk_label_new ("");
+	gtk_misc_set_alignment (GTK_MISC (mem_graph->swappercent_label), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), mem_graph->swappercent_label, 5, 6, 1, 2, 
+			  GTK_FILL, 0, 0, 0);
+			  
 	procdata->mem_graph = mem_graph;
 	gtk_widget_show_all (vbox);
 
@@ -523,12 +526,19 @@ create_sys_view (ProcData *procdata)
 
 	gtk_box_pack_start (GTK_BOX (disk_hbox), scrolled, TRUE, TRUE, 0);
 
-	 
-	model = gtk_tree_store_new (9, GDK_TYPE_PIXBUF, G_TYPE_STRING, 
-					   G_TYPE_STRING, G_TYPE_STRING,
-				       G_TYPE_STRING, G_TYPE_STRING,
-					G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT);
-				       
+
+	model = gtk_tree_store_new (10,
+				    GDK_TYPE_PIXBUF,
+				    G_TYPE_STRING, // device name
+				    G_TYPE_STRING, // directory
+				    G_TYPE_STRING, // type
+				    G_TYPE_STRING, // total
+				    G_TYPE_STRING, // free
+				    G_TYPE_STRING, // used
+				    G_TYPE_FLOAT,
+				    G_TYPE_FLOAT,
+				    G_TYPE_FLOAT);
+
 	disk_tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
 	procdata->disk_list = disk_tree;
 	gtk_container_add (GTK_CONTAINER (scrolled), disk_tree);
@@ -552,7 +562,7 @@ create_sys_view (ProcData *procdata)
 	gtk_tree_view_column_set_resizable (col, TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (disk_tree), col);
 								
-  	for (i = 1; i < 4	; i++) {
+  	for (i = 1; i < 5	; i++) {
   		cell = gtk_cell_renderer_text_new ();
   		col = gtk_tree_view_column_new_with_attributes (titles[i],
 						    		cell,
@@ -568,26 +578,29 @@ create_sys_view (ProcData *procdata)
 	cell = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (col, cell, FALSE);
 	gtk_tree_view_column_set_attributes (col, cell,
-					     "text", 5,
+					     "text", 6,
 					     NULL);
-	gtk_tree_view_column_set_title (col, _(titles[4]));
+	gtk_tree_view_column_set_title (col, titles[5]);
 		
 	
 	cell = procman_cell_renderer_progress_new ();
 	gtk_tree_view_column_pack_start (col, cell, TRUE);
 	gtk_tree_view_column_set_attributes (col, cell,
-					     "value", 6,
+					     "value", 7,
 					     NULL);
 	
 	gtk_tree_view_append_column (GTK_TREE_VIEW (disk_tree), col);
 	gtk_tree_view_column_set_resizable (col, TRUE);
-	gtk_tree_view_column_set_sort_column_id (col, 5);
+	gtk_tree_view_column_set_sort_column_id (col, 6);
 	
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
 					 4, sort_bytes, GINT_TO_POINTER (4), NULL);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
 					 5, sort_bytes, GINT_TO_POINTER (5), NULL);
-	
+	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
+					 6, sort_bytes, GINT_TO_POINTER (6), NULL);
+
+
 	gtk_widget_show_all (disk_box);
   	
   	procman_get_tree_state (procdata->client, disk_tree, "/apps/procman/disktreenew");
@@ -600,7 +613,8 @@ create_sys_view (ProcData *procdata)
 	return vpane;
 }
 
-GtkWidget*
+
+void
 create_main_window (ProcData *procdata)
 {
 	gint width, height;
@@ -633,7 +647,7 @@ create_main_window (ProcData *procdata)
 	gtk_widget_show (tab_label2);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), sys_box, tab_label2);
 	
-	appbar1 = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER);
+	appbar1 = gtk_statusbar_new();
 	gnome_app_set_statusbar (GNOME_APP (app), appbar1);
 	
 	g_signal_connect (G_OBJECT (app), "delete_event",
@@ -669,107 +683,29 @@ create_main_window (ProcData *procdata)
  	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (view1_menu_uiinfo[3].widget),
 							  procdata->config.show_threads);
 	
- 	return app;
+	procdata->app = app;
 
 }
 
-GtkWidget*
-create_simple_view_dialog (ProcData *procdata)
-{
-#if 0
-	GtkWidget *app = NULL;
-	GtkWidget *main_vbox;
-	GtkWidget *scrolled;
-	GtkWidget *vbox;
-	GtkWidget *hbox;
-	GtkWidget *label;
-	GtkWidget *button;
-	GtkWidget *frame;
-	guint key;
-	
-	app = gnome_dialog_new (_("Application Manager"), GNOME_STOCK_BUTTON_CANCEL, NULL);
-	gtk_window_set_policy (GTK_WINDOW (app), FALSE, TRUE, FALSE);
-	gtk_window_set_default_size (GTK_WINDOW (app), 350, 425);
-	
-	main_vbox = GNOME_DIALOG (app)->vbox;
-	
-	frame = gtk_frame_new (_("Running Applications"));
-	gtk_container_set_border_width (GTK_CONTAINER (frame), GNOME_PAD_SMALL);
-	gtk_box_pack_start (GTK_BOX (main_vbox), frame, TRUE, TRUE, 0);
-	
-	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
-	gtk_container_add (GTK_CONTAINER (frame), vbox);
-	
-	scrolled = proctable_new (procdata);
-	if (!scrolled)
-		return NULL;
-	gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
-	
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	
-	button = gtk_button_new ();
-	endprocessbutton = button;
-  	label = gtk_label_new (NULL);
-	key = gtk_label_parse_uline (GTK_LABEL (label), _("_Close Application"));
-	gtk_widget_add_accelerator (button, "clicked",
-				    accel,
-				    key,
-				    GDK_MOD1_MASK,
-				    0);
-	gtk_container_add (GTK_CONTAINER (button), label);
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (button), GNOME_PAD_SMALL);
-	gtk_misc_set_padding (GTK_MISC (GTK_BIN (button)->child), 
-			      GNOME_PAD_SMALL, 1);
-	g_signal_connect (G_OBJECT (button), "clicked",
-			    G_CALLBACK (cb_end_process_button_pressed), procdata);
-
-	g_signal_connect (G_OBJECT (app), "close",
-			    G_CALLBACK (cb_close_simple_dialog), procdata);
-	g_signal_connect (G_OBJECT (procdata->tree), "cursor_activated",
-			    G_CALLBACK (cb_table_selected), procdata);
-			    
-	/* Makes sure everything that should be insensitive is at start */
-	gtk_signal_emit_by_name (G_OBJECT (procdata->tree), "cursor_activated",
-				 -1, NULL);
-				 
-	procdata->timeout = gtk_timeout_add (procdata->config.update_interval,
-			 		     cb_timeout, procdata);
-			 		    
-	
-			    
-	gtk_widget_show_all (main_vbox);
-	gtk_widget_show (app);
-	
-	return app;
-#endif
-	return NULL;
-}
 
 void
-toggle_infoview (ProcData *data)
+toggle_infoview (ProcData *procdata)
 {
-	ProcData *procdata = data;
-	GtkWidget *label;
-	
-	label = infolabel;
-		
-	if (procdata->config.show_more_info == FALSE)
+	if (procdata->config.show_more_info)
+	{
+		gtk_widget_hide (procdata->infobox);
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (infolabel),
+						  _("More _Info >>"));
+	}
+	else
 	{
 		infoview_update (procdata);
 		gtk_widget_show_all (procdata->infobox);
-		procdata->config.show_more_info = TRUE;	
-		gtk_label_set_text_with_mnemonic (GTK_LABEL (label),_(lessinfolabel)); 
- 				
-	}			
-	else
-	{
-		gtk_widget_hide (procdata->infobox);
-		procdata->config.show_more_info = FALSE;
- 		gtk_label_set_text_with_mnemonic (GTK_LABEL (label),_(moreinfolabel)); 
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (infolabel),
+						  _("<< Less _Info"));
 	}
+
+	procdata->config.show_more_info = ! procdata->config.show_more_info;
 }
 
 void
@@ -796,18 +732,16 @@ update_sensitivity (ProcData *data, gboolean sensitivity)
 {
 	gtk_widget_set_sensitive (endprocessbutton, sensitivity);
 	
-	if (!data->config.simple_view) {
-		gtk_widget_set_sensitive (data->infobox, sensitivity);
-		gtk_widget_set_sensitive (edit1_menu_uiinfo[0].widget, sensitivity);
-		gtk_widget_set_sensitive (edit1_menu_uiinfo[1].widget, sensitivity);
-		gtk_widget_set_sensitive (view1_menu_uiinfo[0].widget, sensitivity);
-		gtk_widget_set_sensitive (edit1_menu_uiinfo[2].widget, sensitivity);
-		gtk_widget_set_sensitive (edit1_menu_uiinfo[3].widget, sensitivity);
-		gtk_widget_set_sensitive (edit1_menu_uiinfo[4].widget, sensitivity);
-	}
+	gtk_widget_set_sensitive (data->infobox, sensitivity);
+	gtk_widget_set_sensitive (edit1_menu_uiinfo[0].widget, sensitivity);
+	gtk_widget_set_sensitive (edit1_menu_uiinfo[1].widget, sensitivity);
+	gtk_widget_set_sensitive (view1_menu_uiinfo[0].widget, sensitivity);
+	gtk_widget_set_sensitive (edit1_menu_uiinfo[2].widget, sensitivity);
+	gtk_widget_set_sensitive (edit1_menu_uiinfo[3].widget, sensitivity);
+	gtk_widget_set_sensitive (edit1_menu_uiinfo[4].widget, sensitivity);
 }
 
-void		
+static void		
 cb_toggle_tree (GtkMenuItem *menuitem, gpointer data)
 {
 	ProcData *procdata = data;
@@ -823,7 +757,7 @@ cb_toggle_tree (GtkMenuItem *menuitem, gpointer data)
 	
 }
 
-void		
+static void		
 cb_toggle_threads (GtkMenuItem *menuitem, gpointer data)
 {
 	ProcData *procdata = data;
