@@ -21,6 +21,7 @@
 
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <gconf/gconf-client.h>
 #include <glibtop.h>
 #include <glibtop/close.h>
@@ -504,7 +505,8 @@ main (int argc, char *argv[])
 			    
 	client = gconf_client_get_default ();
 	gconf_client_add_dir(client, "/apps/procman", GCONF_CLIENT_PRELOAD_NONE, NULL);
-	
+
+	gnome_vfs_init ();
 	glibtop_init ();
 	
 	procdata = procman_data_new (client);
@@ -524,6 +526,7 @@ main (int argc, char *argv[])
 	procman_free_data (procdata);
 
 	glibtop_close ();
+	gnome_vfs_shutdown ();
 
 	return 0;
 }
